@@ -1,33 +1,19 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*-
+ * vi:set noexpandtab tabstop=8 shiftwidth=8:
  *
- * Copyright (C) 2013-2017 Richard Hughes <richard@hughsie.com>
+ * Copyright (C) 2013-2018 Richard Hughes <richard@hughsie.com>
+ * Copyright (C) 2014-2018 Kalev Lember <klember@redhat.com>
  *
- * Licensed under the GNU General Public License Version 2
- *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
- * (at your option) any later version.
- *
- * This program is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
+ * SPDX-License-Identifier: GPL-2.0+
  */
 
-#ifndef __GS_APP_H
-#define __GS_APP_H
+#pragma once
 
 #include <glib-object.h>
 #include <gdk/gdk.h>
 #include <gdk-pixbuf/gdk-pixbuf.h>
-#include <appstream-glib.h>
-
-#include "gs-price.h"
+#include <libsoup/soup.h>
+#include <appstream.h>
 
 G_BEGIN_DECLS
 
@@ -43,6 +29,55 @@ struct _GsAppClass
 	gpointer		 padding[30];
 };
 
+/**
+ * GsAppState:
+ * @GS_APP_STATE_UNKNOWN:			Unknown state
+ * @GS_APP_STATE_INSTALLED:			Application is installed
+ * @GS_APP_STATE_AVAILABLE:			Application is available
+ * @GS_APP_STATE_AVAILABLE_LOCAL:		Application is locally available as a file
+ * @GS_APP_STATE_UPDATABLE:			Application is installed and updatable
+ * @GS_APP_STATE_UNAVAILABLE:			Application is referenced, but not available
+ * @GS_APP_STATE_QUEUED_FOR_INSTALL:		Application is queued for install
+ * @GS_APP_STATE_INSTALLING:			Application is being installed
+ * @GS_APP_STATE_REMOVING:			Application is being removed
+ * @GS_APP_STATE_UPDATABLE_LIVE:		Application is installed and updatable live
+ * @GS_APP_STATE_PURCHASABLE:			Application is available for purchasing
+ * @GS_APP_STATE_PURCHASING:			Application is being purchased
+ *
+ * The application state.
+ **/
+typedef enum {
+	GS_APP_STATE_UNKNOWN,				/* Since: 0.2.2 */
+	GS_APP_STATE_INSTALLED,				/* Since: 0.2.2 */
+	GS_APP_STATE_AVAILABLE,				/* Since: 0.2.2 */
+	GS_APP_STATE_AVAILABLE_LOCAL,			/* Since: 0.2.2 */
+	GS_APP_STATE_UPDATABLE,				/* Since: 0.2.2 */
+	GS_APP_STATE_UNAVAILABLE,			/* Since: 0.2.2 */
+	GS_APP_STATE_QUEUED_FOR_INSTALL,		/* Since: 0.2.2 */
+	GS_APP_STATE_INSTALLING,			/* Since: 0.2.2 */
+	GS_APP_STATE_REMOVING,				/* Since: 0.2.2 */
+	GS_APP_STATE_UPDATABLE_LIVE,			/* Since: 0.5.4 */
+	GS_APP_STATE_PURCHASABLE,			/* Since: 0.5.17 */
+	GS_APP_STATE_PURCHASING,			/* Since: 0.5.17 */
+	GS_APP_STATE_LAST  /*< skip >*/
+} GsAppState;
+
+/**
+ * GsAppSpecialKind:
+ * @GS_APP_SPECIAL_KIND_NONE:			No special occupation
+ * @GS_APP_SPECIAL_KIND_OS_UPDATE:		Application represents an OS update
+ *
+ * A special occupation for #GsApp. #AsComponentKind can not represent certain
+ * GNOME Software specific features, like representing a #GsApp as OS updates
+ * which have no associated AppStream entry.
+ * They are represented by a #GsApp of kind %AS_COMPONENT_KIND_GENERIC and a value
+ * from #GsAppSpecialKind. which does not match any AppStream component type.
+ **/
+typedef enum {
+	GS_APP_SPECIAL_KIND_NONE,		/* Since: 40 */
+	GS_APP_SPECIAL_KIND_OS_UPDATE,		/* Since: 40 */
+} GsAppSpecialKind;
+
 /**
  * GsAppKudo:
  * @GS_APP_KUDO_MY_LANGUAGE:		Localised in my language
@@ -53,10 +88,8 @@ struct _GsAppClass
  * @GS_APP_KUDO_INSTALLS_USER_DOCS:	Installs user docs
  * @GS_APP_KUDO_USES_NOTIFICATIONS:	Registers notifications
  * @GS_APP_KUDO_HAS_KEYWORDS:		Has at least 1 keyword
- * @GS_APP_KUDO_USES_APP_MENU:		Uses an AppMenu for navigation
  * @GS_APP_KUDO_HAS_SCREENSHOTS:	Supplies screenshots
  * @GS_APP_KUDO_POPULAR:		Is popular
- * @GS_APP_KUDO_PERFECT_SCREENSHOTS:	Supplies perfect screenshots
  * @GS_APP_KUDO_HIGH_CONTRAST:		Installs a high contrast icon
  * @GS_APP_KUDO_HI_DPI_ICON:		Installs a HiDPI icon
  * @GS_APP_KUDO_SANDBOXED:		Application is sandboxed
@@ -73,18 +106,62 @@ typedef enum {
 	GS_APP_KUDO_INSTALLS_USER_DOCS		= 1 << 5,
 	GS_APP_KUDO_USES_NOTIFICATIONS		= 1 << 6,
 	GS_APP_KUDO_HAS_KEYWORDS		= 1 << 7,
-	GS_APP_KUDO_USES_APP_MENU		= 1 << 8,
 	GS_APP_KUDO_HAS_SCREENSHOTS		= 1 << 9,
 	GS_APP_KUDO_POPULAR			= 1 << 10,
-	GS_APP_KUDO_PERFECT_SCREENSHOTS		= 1 << 12,
 	GS_APP_KUDO_HIGH_CONTRAST		= 1 << 13,
 	GS_APP_KUDO_HI_DPI_ICON			= 1 << 14,
 	GS_APP_KUDO_SANDBOXED			= 1 << 15,
 	GS_APP_KUDO_SANDBOXED_SECURE		= 1 << 16,
-	/*< private >*/
-	GS_APP_KUDO_LAST
+	GS_APP_KUDO_LAST  /*< skip >*/
 } GsAppKudo;
 
+/**
+ * GsAppQuirk:
+ * @GS_APP_QUIRK_NONE:			No special attributes
+ * @GS_APP_QUIRK_PROVENANCE:		Installed by OS vendor
+ * @GS_APP_QUIRK_COMPULSORY:		Cannot be removed
+ * @GS_APP_QUIRK_HAS_SOURCE:		Has a source to allow staying up-to-date
+ * @GS_APP_QUIRK_IS_WILDCARD:		Matches applications from any plugin
+ * @GS_APP_QUIRK_NEEDS_REBOOT:		A reboot is required after the action
+ * @GS_APP_QUIRK_NOT_REVIEWABLE:	The app is not reviewable
+ * @GS_APP_QUIRK_HAS_SHORTCUT:		The app has a shortcut in the system
+ * @GS_APP_QUIRK_NOT_LAUNCHABLE:	The app is not launchable (run-able)
+ * @GS_APP_QUIRK_NEEDS_USER_ACTION:	The component requires some kind of user action
+ * @GS_APP_QUIRK_IS_PROXY:		Is a proxy app that operates on other applications
+ * @GS_APP_QUIRK_REMOVABLE_HARDWARE:	The device is unusable whilst the action is performed
+ * @GS_APP_QUIRK_DEVELOPER_VERIFIED:	The app developer has been verified
+ * @GS_APP_QUIRK_PARENTAL_FILTER:	The app has been filtered by parental controls, and should be hidden
+ * @GS_APP_QUIRK_NEW_PERMISSIONS:	The update requires new permissions
+ * @GS_APP_QUIRK_PARENTAL_NOT_LAUNCHABLE:	The app cannot be run by the current user due to parental controls, and should not be launchable
+ * @GS_APP_QUIRK_HIDE_FROM_SEARCH:	The app should not be shown in search results
+ * @GS_APP_QUIRK_HIDE_EVERYWHERE:	The app should not be shown anywhere (it’s blocklisted)
+ * @GS_APP_QUIRK_DO_NOT_AUTO_UPDATE:	The app should not be automatically updated
+ *
+ * The application attributes.
+ **/
+typedef enum {
+	GS_APP_QUIRK_NONE		= 0,		/* Since: 3.32 */
+	GS_APP_QUIRK_PROVENANCE		= 1 << 0,	/* Since: 3.32 */
+	GS_APP_QUIRK_COMPULSORY		= 1 << 1,	/* Since: 3.32 */
+	GS_APP_QUIRK_HAS_SOURCE		= 1 << 2,	/* Since: 3.32 */
+	GS_APP_QUIRK_IS_WILDCARD	= 1 << 3,	/* Since: 3.32 */
+	GS_APP_QUIRK_NEEDS_REBOOT	= 1 << 4,	/* Since: 3.32 */
+	GS_APP_QUIRK_NOT_REVIEWABLE	= 1 << 5,	/* Since: 3.32 */
+	GS_APP_QUIRK_HAS_SHORTCUT	= 1 << 6,	/* Since: 3.32 */
+	GS_APP_QUIRK_NOT_LAUNCHABLE	= 1 << 7,	/* Since: 3.32 */
+	GS_APP_QUIRK_NEEDS_USER_ACTION	= 1 << 8,	/* Since: 3.32 */
+	GS_APP_QUIRK_IS_PROXY 		= 1 << 9,	/* Since: 3.32 */
+	GS_APP_QUIRK_REMOVABLE_HARDWARE	= 1 << 10,	/* Since: 3.32 */
+	GS_APP_QUIRK_DEVELOPER_VERIFIED	= 1 << 11,	/* Since: 3.32 */
+	GS_APP_QUIRK_PARENTAL_FILTER	= 1 << 12,	/* Since: 3.32 */
+	GS_APP_QUIRK_NEW_PERMISSIONS	= 1 << 13,	/* Since: 3.32 */
+	GS_APP_QUIRK_PARENTAL_NOT_LAUNCHABLE	= 1 << 14,	/* Since: 3.32 */
+	GS_APP_QUIRK_HIDE_FROM_SEARCH	= 1 << 15,	/* Since: 3.32 */
+	GS_APP_QUIRK_HIDE_EVERYWHERE	= 1 << 16,	/* Since: 3.36 */
+	GS_APP_QUIRK_DO_NOT_AUTO_UPDATE	= 1 << 17,	/* Since: 3.36 */
+	GS_APP_QUIRK_LAST  /*< skip >*/
+} GsAppQuirk;
+
 #define	GS_APP_INSTALL_DATE_UNSET		0
 #define	GS_APP_INSTALL_DATE_UNKNOWN		1 /* 1s past the epoch */
 #define	GS_APP_SIZE_UNKNOWABLE			G_MAXUINT64
@@ -103,15 +180,54 @@ typedef enum {
 	GS_APP_QUALITY_LOWEST,
 	GS_APP_QUALITY_NORMAL,
 	GS_APP_QUALITY_HIGHEST,
-	/*< private >*/
-	GS_APP_QUALITY_LAST
+	GS_APP_QUALITY_LAST  /*< skip >*/
 } GsAppQuality;
 
+typedef enum {
+	GS_APP_PERMISSIONS_UNKNOWN 		= 0,
+	GS_APP_PERMISSIONS_NONE			= 1 << 0,
+	GS_APP_PERMISSIONS_NETWORK 		= 1 << 1,
+	GS_APP_PERMISSIONS_SYSTEM_BUS   	= 1 << 2,
+	GS_APP_PERMISSIONS_SESSION_BUS		= 1 << 3,
+	GS_APP_PERMISSIONS_DEVICES 		= 1 << 4,
+	GS_APP_PERMISSIONS_HOME_FULL 		= 1 << 5,
+	GS_APP_PERMISSIONS_HOME_READ		= 1 << 6,
+	GS_APP_PERMISSIONS_FILESYSTEM_FULL	= 1 << 7,
+	GS_APP_PERMISSIONS_FILESYSTEM_READ	= 1 << 8,
+	GS_APP_PERMISSIONS_DOWNLOADS_FULL 	= 1 << 9,
+	GS_APP_PERMISSIONS_DOWNLOADS_READ	= 1 << 10,
+	GS_APP_PERMISSIONS_SETTINGS		= 1 << 11,
+	GS_APP_PERMISSIONS_X11			= 1 << 12,
+	GS_APP_PERMISSIONS_ESCAPE_SANDBOX	= 1 << 13,
+	GS_APP_PERMISSIONS_LAST  /*< skip >*/
+} GsAppPermissions;
+
+#define LIMITED_PERMISSIONS (GS_APP_PERMISSIONS_SETTINGS | \
+			GS_APP_PERMISSIONS_NETWORK | \
+			GS_APP_PERMISSIONS_DOWNLOADS_READ | \
+			GS_APP_PERMISSIONS_DOWNLOADS_FULL)
+#define MEDIUM_PERMISSIONS (LIMITED_PERMISSIONS | \
+			GS_APP_PERMISSIONS_X11)
+
+/**
+ * GS_APP_PROGRESS_UNKNOWN:
+ *
+ * A value returned by gs_app_get_progress() if the app’s progress is unknown
+ * or has a wide confidence interval. Typically this would be represented in the
+ * UI using a pulsing progress bar or spinner.
+ *
+ * Since: 3.38
+ */
+#define GS_APP_PROGRESS_UNKNOWN G_MAXUINT
+
+const gchar 	*gs_app_state_to_string (GsAppState state);
+
 GsApp		*gs_app_new			(const gchar	*id);
 G_DEPRECATED_FOR(gs_app_set_from_unique_id)
 GsApp		*gs_app_new_from_unique_id	(const gchar	*unique_id);
 void		 gs_app_set_from_unique_id	(GsApp		*app,
-						 const gchar	*unique_id);
+						 const gchar	*unique_id,
+						 AsComponentKind kind);
 gchar		*gs_app_to_string		(GsApp		*app);
 void		 gs_app_to_string_append	(GsApp		*app,
 						 GString	*str);
@@ -119,25 +235,28 @@ void		 gs_app_to_string_append	(GsApp		*app,
 const gchar	*gs_app_get_id			(GsApp		*app);
 void		 gs_app_set_id			(GsApp		*app,
 						 const gchar	*id);
-AsAppKind	 gs_app_get_kind		(GsApp		*app);
+AsComponentKind	 gs_app_get_kind		(GsApp		*app);
 void		 gs_app_set_kind		(GsApp		*app,
-						 AsAppKind	 kind);
-AsAppState	 gs_app_get_state		(GsApp		*app);
+						 AsComponentKind kind);
+GsAppState	 gs_app_get_state		(GsApp		*app);
 void		 gs_app_set_state		(GsApp		*app,
-						 AsAppState	 state);
-AsAppScope	 gs_app_get_scope		(GsApp		*app);
+						 GsAppState	 state);
+AsComponentScope gs_app_get_scope		(GsApp		*app);
 void		 gs_app_set_scope		(GsApp		*app,
-						 AsAppScope	 scope);
+						 AsComponentScope scope);
 AsBundleKind	 gs_app_get_bundle_kind		(GsApp		*app);
 void		 gs_app_set_bundle_kind		(GsApp		*app,
 						 AsBundleKind	 bundle_kind);
+GsAppSpecialKind gs_app_get_special_kind	(GsApp		*app);
+void		 gs_app_set_special_kind	(GsApp		*app,
+						 GsAppSpecialKind kind);
 void		 gs_app_set_state_recover	(GsApp		*app);
 guint		 gs_app_get_progress		(GsApp		*app);
 void		 gs_app_set_progress		(GsApp		*app,
 						 guint		 percentage);
 gboolean	gs_app_get_allow_cancel		(GsApp	*app);
 void		gs_app_set_allow_cancel		 (GsApp	*app,
-						  gboolean	can_cancel);
+						  gboolean	allow_cancel);
 const gchar	*gs_app_get_unique_id		(GsApp		*app);
 const gchar	*gs_app_get_branch		(GsApp		*app);
 void		 gs_app_set_branch		(GsApp		*app,
@@ -146,6 +265,9 @@ const gchar	*gs_app_get_name		(GsApp		*app);
 void		 gs_app_set_name		(GsApp		*app,
 						 GsAppQuality	 quality,
 						 const gchar	*name);
+const gchar	*gs_app_get_renamed_from	(GsApp		*app);
+void		 gs_app_set_renamed_from	(GsApp		*app,
+						 const gchar	*renamed_from);
 const gchar	*gs_app_get_source_default	(GsApp		*app);
 void		 gs_app_add_source		(GsApp		*app,
 						 const gchar	*source);
@@ -165,6 +287,9 @@ void		 gs_app_set_project_group	(GsApp		*app,
 const gchar	*gs_app_get_developer_name	(GsApp		*app);
 void		 gs_app_set_developer_name	(GsApp		*app,
 						 const gchar	*developer_name);
+const gchar	*gs_app_get_agreement		(GsApp		*app);
+void		 gs_app_set_agreement		(GsApp		*app,
+						 const gchar	*agreement);
 const gchar	*gs_app_get_version		(GsApp		*app);
 const gchar	*gs_app_get_version_ui		(GsApp		*app);
 void		 gs_app_set_version		(GsApp		*app,
@@ -185,6 +310,14 @@ const gchar	*gs_app_get_url			(GsApp		*app,
 void		 gs_app_set_url			(GsApp		*app,
 						 AsUrlKind	 kind,
 						 const gchar	*url);
+const gchar	*gs_app_get_url_missing		(GsApp		*app);
+void		 gs_app_set_url_missing		(GsApp		*app,
+						 const gchar	*url);
+const gchar	*gs_app_get_launchable		(GsApp		*app,
+						 AsLaunchableKind kind);
+void		 gs_app_set_launchable		(GsApp		*app,
+						 AsLaunchableKind kind,
+						 const gchar	*launchable);
 const gchar	*gs_app_get_license		(GsApp		*app);
 gboolean	 gs_app_get_license_is_free	(GsApp		*app);
 void		 gs_app_set_license		(GsApp		*app,
@@ -196,12 +329,18 @@ void		 gs_app_set_menu_path		(GsApp		*app,
 const gchar	*gs_app_get_origin		(GsApp		*app);
 void		 gs_app_set_origin		(GsApp		*app,
 						 const gchar	*origin);
+const gchar	*gs_app_get_origin_appstream	(GsApp		*app);
+void		 gs_app_set_origin_appstream	(GsApp		*app,
+						 const gchar	*origin_appstream);
 const gchar	*gs_app_get_origin_hostname	(GsApp		*app);
 void		 gs_app_set_origin_hostname	(GsApp		*app,
 						 const gchar	*origin_hostname);
 GPtrArray	*gs_app_get_screenshots		(GsApp		*app);
 void		 gs_app_add_screenshot		(GsApp		*app,
 						 AsScreenshot	*screenshot);
+AsScreenshot	*gs_app_get_action_screenshot	(GsApp		*app);
+void		 gs_app_set_action_screenshot	(GsApp		*app,
+						 AsScreenshot	*screenshot);
 const gchar	*gs_app_get_update_version	(GsApp		*app);
 const gchar	*gs_app_get_update_version_ui	(GsApp		*app);
 void		 gs_app_set_update_version	(GsApp		*app,
@@ -212,22 +351,18 @@ void		 gs_app_set_update_details	(GsApp		*app,
 AsUrgencyKind	 gs_app_get_update_urgency	(GsApp		*app);
 void		 gs_app_set_update_urgency	(GsApp		*app,
 						 AsUrgencyKind	 update_urgency);
-GsApp		*gs_app_get_update_runtime	(GsApp		*app);
-void		 gs_app_set_update_runtime	(GsApp		*app,
-						 GsApp		*runtime);
 const gchar	*gs_app_get_management_plugin	(GsApp		*app);
 void		 gs_app_set_management_plugin	(GsApp		*app,
 						 const gchar	*management_plugin);
-GdkPixbuf	*gs_app_get_pixbuf		(GsApp		*app);
-void		 gs_app_set_pixbuf		(GsApp		*app,
-						 GdkPixbuf	*pixbuf);
-GsPrice		*gs_app_get_price		(GsApp		*app);
-void		 gs_app_set_price		(GsApp		*app,
-						 gdouble	 amount,
-						 const gchar	*currency);
+GIcon		*gs_app_get_icon_for_size	(GsApp		*app,
+						 guint		 size,
+						 guint		 scale,
+						 const gchar	*fallback_icon_name);
 GPtrArray	*gs_app_get_icons		(GsApp		*app);
 void		 gs_app_add_icon		(GsApp		*app,
-						 AsIcon		*icon);
+						 GIcon		*icon);
+void		 gs_app_remove_all_icons	(GsApp		*app);
+gboolean	 gs_app_get_use_drop_shadow	(GsApp		*app);
 GFile		*gs_app_get_local_file		(GsApp		*app);
 void		 gs_app_set_local_file		(GsApp		*app,
 						 GFile		*local_file);
@@ -258,46 +393,51 @@ void		 gs_app_add_review		(GsApp		*app,
 						 AsReview	*review);
 void		 gs_app_remove_review		(GsApp		*app,
 						 AsReview	*review);
-GPtrArray	*gs_app_get_provides		(GsApp		*app);
-void		 gs_app_add_provide		(GsApp		*app,
-						 AsProvide	*provide);
+GPtrArray	*gs_app_get_provided		(GsApp		*app);
+AsProvided	*gs_app_get_provided_for_kind	(GsApp		*app,
+						 AsProvidedKind kind);
+void		 gs_app_add_provided_item	(GsApp		*app,
+						 AsProvidedKind kind,
+						 const gchar	*item);
 guint64		 gs_app_get_size_installed	(GsApp		*app);
 void		 gs_app_set_size_installed	(GsApp		*app,
 						 guint64	 size_installed);
 guint64		 gs_app_get_size_download	(GsApp		*app);
 void		 gs_app_set_size_download	(GsApp		*app,
 						 guint64	 size_download);
-GPtrArray	*gs_app_get_addons		(GsApp		*app);
-void		 gs_app_add_addon		(GsApp		*app,
-						 GsApp		*addon);
-GPtrArray	*gs_app_get_related		(GsApp		*app);
 void		 gs_app_add_related		(GsApp		*app,
 						 GsApp		*app2);
-GPtrArray	*gs_app_get_history		(GsApp		*app);
+void		 gs_app_add_addon		(GsApp		*app,
+						 GsApp		*addon);
 void		 gs_app_add_history		(GsApp		*app,
 						 GsApp		*app2);
 guint64		 gs_app_get_install_date	(GsApp		*app);
 void		 gs_app_set_install_date	(GsApp		*app,
 						 guint64	 install_date);
+guint64		 gs_app_get_release_date	(GsApp		*app);
+void		 gs_app_set_release_date	(GsApp		*app,
+						 guint64	 release_date);
 GPtrArray	*gs_app_get_categories		(GsApp		*app);
 void		 gs_app_set_categories		(GsApp		*app,
 						 GPtrArray	*categories);
-GPtrArray	*gs_app_get_key_colors		(GsApp		*app);
+GArray		*gs_app_get_key_colors		(GsApp		*app);
 void		 gs_app_set_key_colors		(GsApp		*app,
-						 GPtrArray	*key_colors);
+						 GArray		*key_colors);
 void		 gs_app_add_key_color		(GsApp		*app,
 						 GdkRGBA	*key_color);
+void            gs_app_set_is_update_downloaded (GsApp         *app,
+                                                 gboolean       is_update_downloaded);
+gboolean        gs_app_get_is_update_downloaded (GsApp         *app);
 gboolean	 gs_app_has_category		(GsApp		*app,
 						 const gchar	*category);
 void		 gs_app_add_category		(GsApp		*app,
 						 const gchar	*category);
 gboolean	 gs_app_remove_category		(GsApp		*app,
 						 const gchar	*category);
-GPtrArray	*gs_app_get_keywords		(GsApp		*app);
-void		 gs_app_set_keywords		(GsApp		*app,
-						 GPtrArray	*keywords);
 void		 gs_app_add_kudo		(GsApp		*app,
 						 GsAppKudo	 kudo);
+void		 gs_app_remove_kudo		(GsApp		*app,
+						 GsAppKudo	 kudo);
 gboolean	 gs_app_has_kudo		(GsApp		*app,
 						 GsAppKudo	 kudo);
 guint64		 gs_app_get_kudos		(GsApp		*app);
@@ -310,15 +450,31 @@ void		 gs_app_set_match_value		(GsApp		*app,
 guint		 gs_app_get_match_value		(GsApp		*app);
 
 gboolean	 gs_app_has_quirk		(GsApp		*app,
-						 AsAppQuirk	 quirk);
+						 GsAppQuirk	 quirk);
 void		 gs_app_add_quirk		(GsApp		*app,
-						 AsAppQuirk	 quirk);
+						 GsAppQuirk	 quirk);
 void		 gs_app_remove_quirk		(GsApp		*app,
-						 AsAppQuirk	 quirk);
+						 GsAppQuirk	 quirk);
 gboolean	 gs_app_is_installed		(GsApp		*app);
 gboolean	 gs_app_is_updatable		(GsApp		*app);
+gchar		*gs_app_get_origin_ui		(GsApp		*app);
+void		 gs_app_set_origin_ui		(GsApp		*app,
+						 const gchar	*origin_ui);
+gchar		*gs_app_get_packaging_format	(GsApp		*app);
+void		 gs_app_subsume_metadata	(GsApp		*app,
+						 GsApp		*donor);
+GsAppPermissions gs_app_get_permissions		(GsApp		*app);
+void		 gs_app_set_permissions		(GsApp		*app,
+						 GsAppPermissions permissions);
+GsAppPermissions gs_app_get_update_permissions	(GsApp		*app);
+void		 gs_app_set_update_permissions	(GsApp		*app,
+						 GsAppPermissions update_permissions);
+GPtrArray	*gs_app_get_version_history	(GsApp		*app);
+void		 gs_app_set_version_history	(GsApp		*app,
+						 GPtrArray	*version_history);
+void		gs_app_ensure_icons_downloaded	(GsApp		*app,
+						 SoupSession	*soup_session,
+						 guint		 maximum_icon_size,
+						 GCancellable	*cancellable);
+
 G_END_DECLS
-
-#endif /* __GS_APP_H */
-
-/* vim: set noexpandtab: */
